from .client_proxy import ClientProxy
from .role import Role


class RolesProxy(ClientProxy):
    @property
    def _resource_class(self):
        return Role

    def create(self, attributes=None, **kwargs):
        """Creates a Role with given attributes."""

        return super(RolesProxy, self).create(None, attributes)



from .content_type_resource_proxy import ContentTypeResourceProxy
from .editor_interfaces_proxy import EditorInterfacesProxy


"""
contentful.content_type_editor_interfaces_proxy
~~~~~~~~~~~~~~~~~~~~~~~

This module implements the ContentTypeEditorInterfacesProxy class.

API Reference: https://www.contentful.com/developers/docs/references/content-management-api/#/reference/editor-interface

:copyright: (c) 2017 by Contentful GmbH.
:license: MIT, see LICENSE for more details.
"""


class ContentTypeEditorInterfacesProxy(ContentTypeResourceProxy):
    """
    API Reference: https://www.contentful.com/developers/docs/references/content-management-api/#/reference/editor-interface
    """

    def _resource_proxy_class(self):
        return EditorInterfacesProxy

    def all(self):
        """Gets the Default Editor Interface"""

        return self.proxy.all()

    def find(self, *args, **kwargs):
        """Gets the Default Editor Interface"""

        return self.proxy.find()

    def create(self, **kwargs):
        """Not Supported"""

        raise Exception("Not Supported")

    def delete(self, *args, **kwargs):
        """Not Supported"""

        raise Exception("Not Supported")



from .space_resource_proxy import SpaceResourceProxy
from .locales_proxy import LocalesProxy


"""
contentful.space_locales_proxy
~~~~~~~~~~~~~~~~~~~~~~~

This module implements the SpaceLocalesProxy class.

API Reference: https://www.contentful.com/developers/docs/references/content-management-api/#/reference/locales

:copyright: (c) 2017 by Contentful GmbH.
:license: MIT, see LICENSE for more details.
"""


class SpaceLocalesProxy(SpaceResourceProxy):
    """
    API Reference: https://www.contentful.com/developers/docs/references/content-management-api/#/reference/locales
    """

    def _resource_proxy_class(self):
        return LocalesProxy



try:
    import simplejson as json
except ImportError:
    import json

import dateutil.parser
from .utils import unicode_class

"""
contentful.content_type_field_types
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

This module implements the Field Coercion classes.

:copyright: (c) 2017 by Contentful GmbH.
:license: MIT, see LICENSE for more details.
"""


class BasicField(object):
    """Base Coercion Class"""

    def __init__(self, items=None):
        self._items = items

    def coerce(self, value):
        """Just returns the value."""

        return value

    def __repr__(self):
        return "<{0}>".format(
            self.__class__.__name__
        )


class SymbolField(BasicField):
    """Symbol Coercion Class"""

    def coerce(self, value):
        """Coerces value to str"""

        return unicode_class()(value)


class TextField(BasicField):
    """Text Coercion Class"""

    def coerce(self, value):
        """Coerces value to str"""

        return unicode_class()(value)


class IntegerField(BasicField):
    """Integer Coercion Class"""

    def coerce(self, value):
        """Coerces value to int"""

        return int(value) if value is not None else None


class NumberField(BasicField):
    """Number Coercion Class"""

    def coerce(self, value):
        """Coerces value to float"""

        return float(value) if value is not None else None


class DateField(BasicField):
    """Date Coercion Class"""

    def coerce(self, value):
        """Coerces ISO8601 date to :class:`datetime.datetime` object."""

        return dateutil.parser.parse(value)


class BooleanField(BasicField):
    """Boolean Coercion Class"""

    def coerce(self, value):
        """Coerces value to boolean"""

        return bool(value)


class LinkField(BasicField):
    """
    LinkField

    Nothing should be done here as include resolution is handled within
    entries due to depth handling (explained within Entry).
    Only present as a placeholder for proper resolution within ContentType.
    """
    pass


class ArrayField(BasicField):
    """Array Coercion Class

    Coerces items in collection with it's proper Coercion Class.
    """

    def __init__(self, items=None):
        super(ArrayField, self).__init__(items)
        self._coercion = self._get_coercion()

    def coerce(self, value):
        """Coerces array items with proper coercion."""

        result = []
        for v in value:
            result.append(self._coercion.coerce(v))
        return result

    def _get_coercion(self):
        return globals()["{0}Field".format(self._items.get('type'))]()


class ObjectField(BasicField):
    """Object Coercion Class"""

    def coerce(self, value):
        """Coerces value to JSON"""

        return json.loads(json.dumps(value))


class LocationField(BasicField):
    """Location Coercion Class"""

    def coerce(self, value):
        """Coerces value to Location Hash"""

        return {
            'lat': float(value.get('lat', value.get('latitude'))),
            'lon': float(value.get('lon', value.get('longitude')))
        }



from .resource import Resource


"""
contentful.Snapshot
~~~~~~~~~~~~~~~~~~~~~~~

This module implements the Snapshot class.

API Reference: https://www.contentful.com/developers/docs/references/content-delivery-api/#/reference/content-types

:copyright: (c) 2016 by Contentful GmbH.
:license: MIT, see LICENSE for more details.
"""


class Snapshot(Resource):
    """
    API Reference: https://www.contentful.com/developers/docs/references/content-delivery-api/#/reference/content-types
    """

    def __init__(self, item, **kwargs):
        super(Snapshot, self).__init__(item, **kwargs)

        from .entry import Entry
        self.snapshot = Entry(item['snapshot'], **kwargs)

    @classmethod
    def base_url(klass, space_id, entry_id, resource_id=None):
        """Returns the URI for the Resource"""

        return "spaces/{0}/entries/{1}/snapshots/{2}".format(
            space_id,
            entry_id,
            resource_id if resource_id is not None else ''
        )

    def to_json(self):
        """Returns the JSON Representation of the Resource"""

        result = super(Snapshot, self).to_json()
        result.update({
            'snapshot': self.snapshot.to_json(),
        })
        return result

    def __repr__(self):
        return "<Snapshot id='{0}'>".format(
            self.sys.get('id', '')
        )

    def save(self):
        """Not Supported"""

        raise Exception("Not Supported")

    def update(self, _attributes=None):
        """Not Supported"""

        raise Exception("Not Supported")



from .resource import Resource


"""
contentful.Webhook
~~~~~~~~~~~~~~~~~~~~~~~

This module implements the Webhook class.

API Reference: https://www.contentful.com/developers/docs/references/content-management-api/#/reference/webhooks

:copyright: (c) 2017 by Contentful GmbH.
:license: MIT, see LICENSE for more details.
"""


class Webhook(Resource):
    """
    API Reference: https://www.contentful.com/developers/docs/references/content-management-api/#/reference/webhooks
    """

    def __init__(self, item, **kwargs):
        super(Webhook, self).__init__(item, **kwargs)
        self.name = item.get('name', '')
        self.url = item.get('url', '')
        self.topics = item.get('topics', [])
        self.http_basic_username = item.get('httpBasicUsername', '')
        self.headers = item.get('headers', [])

    @classmethod
    def update_attributes_map(klass):
        """Defines keys and default values for non-generic attributes"""

        return {
            'name': '',
            'url': '',
            'topics': [],
            'http_basic_username': '',
            'headers': []
        }

    @classmethod
    def create_attributes(klass, attributes, previous_object=None):
        """Attributes for resource creation"""

        result = super(Webhook, klass).create_attributes(attributes, previous_object)

        if 'topics' not in result:
            raise Exception("Topics ('topics') must be provided for this operation.")
        return result

    def to_json(self):
        """Returns the JSON Representation of the Resource"""

        result = super(Webhook, self).to_json()
        result.update({
            'name': self.name,
            'url': self.url,
            'topics': self.topics,
            'httpBasicUsername': self.http_basic_username,
            'headers': self.headers
        })
        return result

    def __repr__(self):
        return "<Webhook[{0}] id='{1}'>".format(
            self.name,
            self.sys.get('id', '')
        )



from .client_proxy import ClientProxy
from .api_key import ApiKey


"""
contentful.api_keys_proxy
~~~~~~~~~~~~~~~~~~~~~~~

This module implements the ApiKeysProxy class.

API Reference: https://www.contentful.com/developers/docs/references/content-management-api/#/reference/api-keys

:copyright: (c) 2017 by Contentful GmbH.
:license: MIT, see LICENSE for more details.
"""


class ApiKeysProxy(ClientProxy):
    @property
    def _resource_class(self):
        return ApiKey

    def create(self, attributes=None, **kwargs):
        """Creates an API Key with given attributes."""

        return super(ApiKeysProxy, self).create(None, attributes)



"""
contentful.errors
~~~~~~~~~~~~~~~~~

This module implements the Error classes.

API Reference: https://www.contentful.com/developers/docs/references/content-delivery-api/#/introduction/errors

:copyright: (c) 2016 by Contentful GmbH.
:license: MIT, see LICENSE for more details.
"""


class HTTPError(Exception):
    """
    Base HTTP Error Class
    """

    def __init__(self, response):
        super(HTTPError, self).__init__(response.json()['message'])
        self.response = response
        self.status_code = response.status_code


class NotFoundError(HTTPError):
    """
    404
    """
    pass


class BadRequestError(HTTPError):
    """
    400
    """
    pass


class AccessDeniedError(HTTPError):
    """
    403
    """
    pass


class UnauthorizedError(HTTPError):
    """
    401
    """
    pass


class VersionMismatchError(HTTPError):
    """
    409
    """
    def __init__(self, response):
        self.response = response
        self.status_code = response.status_code
        self.message = 'Version mismatch error. The version you specified was incorrect. This may be due to someone else editing the content.'

    def __repr__(self):
        return self.message

    __str__ = __repr__


class RateLimitExceededError(HTTPError):
    """
    429
    """
    pass


class ServerError(HTTPError):
    """
    500
    """
    pass


class ServiceUnavailableError(HTTPError):
    """
    503
    """
    pass


def get_error(response):
    """Gets Error by HTTP Status Code"""

    errors = {
        400: BadRequestError,
        401: UnauthorizedError,
        403: AccessDeniedError,
        404: NotFoundError,
        409: VersionMismatchError,
        429: RateLimitExceededError,
        500: ServerError,
        503: ServiceUnavailableError
    }

    error_class = HTTPError
    if response.status_code in errors:
        error_class = errors[response.status_code]

    return error_class(response)



from .client_proxy import ClientProxy
from .snapshot import Snapshot


class SnapshotsProxy(ClientProxy):
    def __init__(self, client, space_id, entry_id):
        super(SnapshotsProxy, self).__init__(client, space_id)
        self.entry_id = entry_id

    @property
    def _resource_class(self):
        return Snapshot

    def create(*args, **kwargs):
        """Not Supported"""

        raise Exception("Not Supported")

    def delete(*args, **kwargs):
        """Not Supported"""

        raise Exception("Not Supported")

    def _url(self, resource_id='', **kwargs):
        return self._resource_class.base_url(
            self.space_id,
            self.entry_id,
            resource_id=resource_id
        )

    def __repr__(self):
        return "<{0} space_id='{1}' entry_id='{2}'>".format(
            self.__class__.__name__,
            self.space_id,
            self.entry_id
        )



from .resource import Resource


"""
contentful.api_key
~~~~~~~~~~~~~~~~~~~~~~~

This module implements the ApiKey class.

API Reference: https://www.contentful.com/developers/docs/references/content-management-api/#/reference/api-keys

:copyright: (c) 2017 by Contentful GmbH.
:license: MIT, see LICENSE for more details.
"""


class ApiKey(Resource):
    """
    API Reference: https://www.contentful.com/developers/docs/references/content-management-api/#/reference/api-keys
    """

    def __init__(self, item, **kwargs):
        super(ApiKey, self).__init__(item, **kwargs)
        self.name = item.get('name', '')
        self.description = item.get('description', '')
        self.access_token = item.get('accessToken', '')

    @classmethod
    def create_attributes(klass, attributes, previous_object=None):
        """Attributes for resource creation"""

        if previous_object is not None:
            return {
                'name': attributes.get('name', previous_object.name),
                'description': attributes.get('description', previous_object.description)
            }
        return {
            'name': attributes.get('name', ''),
            'description': attributes.get('description', '')
        }

    @classmethod
    def update_attributes_map(klass):
        """Defines keys and default values for non-generic attributes"""

        return {
            'name': '',
            'description': '',
            'access_token': ''
        }

    def to_json(self):
        """Returns the JSON Representation of the Resource"""

        result = super(ApiKey, self).to_json()
        result.update({
            'name': self.name,
            'description': self.description,
            'accessToken': self.access_token
        })
        return result

    def __repr__(self):
        return "<ApiKey[{0}] id='{1}'>".format(
            self.name,
            self.sys.get('id', '')
        )



from .client_proxy import ClientProxy
from .content_type import ContentType


"""
contentful.content_types_proxy
~~~~~~~~~~~~~~~~~~~~~~~

This module implements the ContentTypesProxy class.

API Reference: https://www.contentful.com/developers/docs/references/content-management-api/#/reference/editor-interface

:copyright: (c) 2017 by Contentful GmbH.
:license: MIT, see LICENSE for more details.
"""


class ContentTypesProxy(ClientProxy):
    """
    API Reference: https://www.contentful.com/developers/docs/references/content-management-api/#/reference/content-types/content-type
    """

    @property
    def _resource_class(self):
        return ContentType

    def all_published(self):
        """
        Gets all the Published Content Types for a Space.

        API Reference: https://www.contentful.com/developers/docs/references/content-management-api/#/reference/content-types/activated-content-type-collection
        """

        return self.client._get(
            self._url(public=True)
        )



from .entry import Entry
from .asset import Asset
from .space import Space
from .content_type import ContentType
from .webhook import Webhook
from .locale import Locale
from .role import Role
from .editor_interface import EditorInterface
from .api_key import ApiKey
from .snapshot import Snapshot
from .upload import Upload
from .array import Array


"""
contentful.resource_builder
~~~~~~~~~~~~~~~~~~~~~~~~~~~

This module implements the Resource Builder class.

:copyright: (c) 2016 by Contentful GmbH.
:license: MIT, see LICENSE for more details.
"""


class ResourceBuilder(object):
    """Creates objects of the proper Resource Type"""

    def __init__(self, client, default_locale, json):
        self.client = client
        self.default_locale = default_locale
        self.json = json

    def build(self):
        """Creates the objects from the JSON response"""

        if self.json['sys']['type'] == 'Array':
            return self._build_array()
        return self._build_item(self.json)

    def _build_array(self):
        return Array(self.json, [self._build_item(item) for item in self.json['items']])

    def _build_item(self, item):
        buildables = {
            'Entry': Entry,
            'Asset': Asset,
            'ContentType': ContentType,
            'Space': Space,
            'ApiKey': ApiKey,
            'Locale': Locale,
            'EditorInterface': EditorInterface,
            'WebhookDefinition': Webhook,
            'Role': Role,
            'Snapshot': Snapshot,
            'Upload': Upload
        }

        item_type = item['sys']['type']

        if item_type in buildables:
            return buildables[item_type](
                item,
                default_locale=self.default_locale,
                client=self.client
            )



from .resource import Resource
from .space_content_types_proxy import SpaceContentTypesProxy
from .space_entries_proxy import SpaceEntriesProxy
from .space_assets_proxy import SpaceAssetsProxy
from .space_webhooks_proxy import SpaceWebhooksProxy
from .space_roles_proxy import SpaceRolesProxy
from .space_locales_proxy import SpaceLocalesProxy
from .space_api_keys_proxy import SpaceApiKeysProxy
from .space_uploads_proxy import SpaceUploadsProxy


"""
contentful.space
~~~~~~~~~~~~~~~~

This module implements the Space class.

API Reference: https://www.contentful.com/developers/docs/references/content-management-api/#/reference/spaces

:copyright: (c) 2017 by Contentful GmbH.
:license: MIT, see LICENSE for more details.
"""


class Space(Resource):
    """
    API Reference: https://www.contentful.com/developers/docs/references/content-management-api/#/reference/spaces
    """

    def __init__(self, item, **kwargs):
        super(Space, self).__init__(item, **kwargs)
        self.name = item.get('name', '')

    @classmethod
    def base_url(klass, space_id=None, **kwargs):
        """Returns the URI for the Resource"""

        if space_id is None:
            space_id = ''
        return "spaces/{0}".format(space_id)

    @classmethod
    def create_headers(klass, attributes):
        """Headers for resource creation"""

        if 'organization_id' in attributes:
            return {'x-contentful-organization': attributes['organization_id']}
        return {}

    @classmethod
    def create_attributes(klass, attributes, previous_object=None):
        """Attributes for resource creation"""

        if previous_object is not None:
            return {'name': attributes.get('name', previous_object.name)}
        return {
            'name': attributes.get('name', ''),
            'defaultLocale': attributes['default_locale']
        }

    @classmethod
    def update_attributes_map(klass):
        """Defines keys and default values for non-generic attributes"""

        return {
            'name': '',
            'default_locale': ''
        }

    def update(self, attributes=None):
        """Updates the Resource with attributes"""

        if attributes is None:
            attributes = {}

        headers = self.__class__.create_headers(attributes)
        headers.update(self._update_headers())

        result = self._client._put(
            self.__class__.base_url(
                self.sys['id']
            ),
            self.__class__.create_attributes(attributes, self),
            headers=headers
        )

        self._update_from_resource(result)

        return self

    def reload(self):
        """Reloads the Resource"""

        result = self._client._get(
            self.__class__.base_url(
                self.sys['id']
            )
        )

        self._update_from_resource(result)

        return self

    def delete(self):
        """Deletes the Space"""

        return self._client._delete(
            self.__class__.base_url(
                self.sys['id']
            )
        )

    def to_json(self):
        """Returns the JSON Representation of the Resource"""

        result = super(Space, self).to_json()
        result.update({'name': self.name})
        return result

    def content_types(self):
        """Provides access to Content Type management methods

        API Reference: https://www.contentful.com/developers/docs/references/content-management-api/#/reference/content-types

        :return: :class:`SpaceContentTypesProxy <contentful_management.space_content_types_proxy.SpaceContentTypesProxy>` object.
        :rtype: contentful.space_content_types_proxy.SpaceEntriesProxy

        Usage:

            >>> space_content_types_proxy = space.content_types()
            <SpaceContentTypesProxy space_id="cfexampleapi">
        """
        return SpaceContentTypesProxy(self._client, self.id)

    def entries(self):
        """Provides access to Entries management methods

        API Reference: https://www.contentful.com/developers/docs/references/content-management-api/#/reference/entries

        :return: :class:`SpaceEntriesProxy <contentful_management.space_entries_proxy.SpaceEntriesProxy>` object.
        :rtype: contentful.space_entries_proxy.SpaceEntriesProxy

        Usage:

            >>> space_entries_proxy = space.entries()
            <SpaceEntriesProxy space_id="cfexampleapi">
        """
        return SpaceEntriesProxy(self._client, self.id)

    def assets(self):
        """Provides access to Assets management methods

        API Reference: https://www.contentful.com/developers/docs/references/content-management-api/#/reference/assets

        :return: :class:`SpaceAssetsProxy <contentful_management.space_assets_proxy.SpaceAssetsProxy>` object.
        :rtype: contentful.space_assets_proxy.SpaceAssetsProxy

        Usage:

            >>> space_assets_proxy = space.assets()
            <SpaceAssetsProxy space_id="cfexampleapi">
        """
        return SpaceAssetsProxy(self._client, self.id)

    def locales(self):
        """Provides access to Locale management methods

        API Reference: https://www.contentful.com/developers/docs/references/content-management-api/#/reference/locales

        :return: :class:`SpaceLocalesProxy <contentful_management.space_locales_proxy.SpaceLocalesProxy>` object.
        :rtype: contentful.space_locales_proxy.SpaceLocalesProxy

        Usage:

            >>> space_locales_proxy = space.locales()
            <SpaceLocalesProxy space_id="cfexampleapi">
        """
        return SpaceLocalesProxy(self._client, self.id)

    def webhooks(self):
        """Provides access to Webhook management methods

        API Reference: https://www.contentful.com/developers/docs/references/content-management-api/#/reference/webhooks

        :return: :class:`SpaceWebhooksProxy <contentful_management.space_webhooks_proxy.SpaceWebhooksProxy>` object.
        :rtype: contentful.space_webhooks_proxy.SpaceWebhooksProxy

        Usage:

            >>> space_webhooks_proxy = space.webhooks()
            <SpaceWebhooksProxy space_id="cfexampleapi">
        """
        return SpaceWebhooksProxy(self._client, self.id)

    def roles(self):
        """Provides access to Role management methods

        API Reference: https://www.contentful.com/developers/docs/references/content-management-api/#/reference/roles

        :return: :class:`SpaceRolesProxy <contentful_management.space_roles_proxy.SpaceRolesProxy>` object.
        :rtype: contentful.space_roles_proxy.SpaceRolesProxy

        Usage:

            >>> space_roles_proxy = space.roles()
            <SpaceRolesProxy space_id="cfexampleapi">
        """
        return SpaceRolesProxy(self._client, self.id)

    def api_keys(self):
        """Provides access to Api Key management methods

        API Reference: https://www.contentful.com/developers/docs/references/content-management-api/#/reference/api-keys

        :return: :class:`SpaceApiKeysProxy <contentful_management.space_api_keys_proxy.SpaceApiKeysProxy>` object.
        :rtype: contentful.space_api_keys_proxy.SpaceApiKeysProxy

        Usage:

            >>> space_api_keys_proxy = space.api_keys()
            <SpaceApiKeysProxy space_id="cfexampleapi">
        """
        return SpaceApiKeysProxy(self._client, self.id)

    def uploads(self):
        """Provides access to Upload management methods

        API Reference: https://www.contentful.com/developers/docs/references/content-management-api/#/reference/uploads-keys

        :return: :class:`SpaceApiKeysProxy <contentful_management.space_api_keys_proxy.SpaceApiKeysProxy>` object.
        :rtype: contentful.space_api_keys_proxy.SpaceApiKeysProxy

        Usage:

            >>> space_uploads_proxy = space.uploads()
            <SpaceUploadsProxy space_id="cfexampleapi">
        """
        return SpaceUploadsProxy(self._client, self.id)

    def __repr__(self):
        return "<Space[{0}] id='{1}'>".format(
            self.name,
            self.id
        )
